/*
 * Copyright (c) 2016, NVIDIA CORPORATION. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __NVIDIA_DRM_CRTC_H__
#define __NVIDIA_DRM_CRTC_H__

#include "nvidia-drm-conftest.h"

#if defined(NV_DRM_ATOMIC_MODESET_AVAILABLE)

#include <drm/drmP.h>
#include "nvtypes.h"
#include "nvkms-kapi.h"

struct nvidia_drm_crtc
{
    NvU32 head;

#if defined(NV_DRM_ATOMIC_MODESET_NONBLOCKING_COMMIT_AVAILABLE)
    /**
     * @flip_list:
     *
     * List of flips pending to get proccessed by __nv_drm_handle_flip_event().
     * Protected by @flip_lock.
     */
    struct list_head flip_list;

    /**
     * @flip_lock:
     *
     * Spinlock to protect @flip_list.
     */
    spinlock_t flip_lock;
#else
    atomic_t has_pending_commit;
    atomic_t has_pending_flip_event;
#endif

    struct drm_crtc base;
};

#define DRM_CRTC_TO_NV_CRTC(__crtc) \
    container_of(__crtc, struct nvidia_drm_crtc, base)

struct drm_crtc *nvidia_drm_add_crtc(struct drm_device *dev, NvU32 head);

int nvidia_drm_get_crtc_crc32(struct drm_device *dev,
                              void *data,
                              struct drm_file *file_priv);

#endif /* NV_DRM_ATOMIC_MODESET_AVAILABLE */

#endif /* __NVIDIA_DRM_CRTC_H__ */
